<link rel="icon" href="{{ 'assets/favicon-light.svg' | url }}" media="(prefers-color-scheme: light)" type="image/svg+xml" />
<link rel="icon" href="{{ 'assets/favicon-dark.svg' | url }}" media="(prefers-color-scheme: dark)" type="image/svg+xml" />
<link rel="icon" href="{{ 'assets/favicon.ico' | url }}" />
<link rel="manifest" href="{{ 'assets/site.webmanifest' | url }}" />
